package com.hncy.file.manage.spring.boot.starter.oss.manager;

import cn.hutool.core.io.FileUtil;
import com.aliyun.oss.HttpMethod;
import com.hncy.file.manage.spring.boot.starter.core.exception.AccessRoleNotConfiguredException;
import com.hncy.file.manage.spring.boot.starter.core.exception.ResourceAccessDeniedException;
import com.hncy.file.manage.spring.boot.starter.core.manager.ResourceManager;
import com.hncy.file.manage.spring.boot.starter.oss.common.OSSClient;
import com.hncy.file.manage.spring.boot.starter.oss.common.OSSClientRegistry;
import com.hncy.file.manage.spring.boot.starter.oss.constant.OSSConstant;
import com.hncy.file.manage.spring.boot.starter.oss.properties.OSSRoleProperties;
import com.hncy.file.manage.spring.boot.starter.oss.resource.OSSResource;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: ying.dong
 * @Date: 2021/5/24 17:47
 */
public abstract class OSSManager implements ResourceManager<OSSResource> {

    private final OSSClientRegistry ossClientRegistry;

    public OSSManager(OSSClientRegistry ossClientRegistry) {
        this.ossClientRegistry = ossClientRegistry;
    }

    protected OSSClientRegistry getOssClientRegistry() {
        return ossClientRegistry;
    }

    /**
     * 获取OSS Client
     * @param role
     * @return
     */
    protected OSSClient getOSSClient(String role) {
        return getOssClientRegistry().get(role)
                .orElseThrow(() -> new AccessRoleNotConfiguredException(role));
    }

    /**
     * 获取文件名
     * @param roleProperties
     * @param resource
     * @return
     */
    protected String getFileName(OSSRoleProperties roleProperties, OSSResource resource) {
        return getPath(roleProperties.getDirectory(), resource.getDirectory(), resource.getName());
    }

    /**
     *
     * @param roleDirectory
     * @param customDirectory
     * @param fileName
     * @return
     */
    protected String getPath(@NonNull String roleDirectory, @Nullable String customDirectory, @Nullable String fileName) {
        List<String> pathList = new ArrayList<>();
        pathList.add(OSSConstant.OSS_SEPARATOR);
        pathList.add(roleDirectory);
        if (!StringUtils.isEmpty(customDirectory)) {
            pathList.add(customDirectory);
        }
        if (!StringUtils.isEmpty(fileName)) {
            pathList.add(fileName);
        }
        String[] paths = new String[pathList.size()];
        String rootPath = FileUtil.file(pathList.toArray(paths)).getPath();
        return rootPath.substring(1);
    }

    /**
     * 初步校验权限
     * @param role
     * @param roleProperties
     * @param httpMethod
     */
    protected void permissionValidated(String role, OSSRoleProperties roleProperties, HttpMethod httpMethod) {
        //通常情况下，我们只会用到GET、PUT、POST、DELETE，因此本处只校验这几种
        if (!roleProperties.getWriteable() && httpMethod != HttpMethod.GET) {
            throw new ResourceAccessDeniedException(
                    "writable access denied for role [" + role + "], bucket [" + roleProperties.getBucket() + "], " +
                            "directory: [" + roleProperties.getDirectory() + "]");
        }

        if (!roleProperties.getReadable() && httpMethod == HttpMethod.GET) {
            throw new ResourceAccessDeniedException(
                    "readable access denied for role [" + role + "], bucket [" + roleProperties.getBucket() + "], " +
                            "directory: [" + roleProperties.getDirectory() + "]");
        }
    }
}
