package com.hncy.jackson.spring.boot.starter.core;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.fasterxml.jackson.databind.type.MapType;
import com.hncy.jackson.spring.boot.starter.exception.JSONException;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Supplier;

/**
 * @Author: ying.dong
 * @Date: 2021/4/22 9:50
 */
public class JSONObject {


    private static final JSONObject jsonObject = new JSONObject(new ObjectMapper());

    private ObjectMapper objectMapper;

    private JSONObject(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    public static JSONObject build(ObjectMapper objectMapper) {
        jsonObject.setObjectMapper(objectMapper);
        return jsonObject;
    }

    public static JSONObject getInstance() {
        return jsonObject;
    }

    private void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }


    /**
     * json字符串转对象
     * @param json
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T parseObject(String json, Class<T> clazz) {
        if (StringUtils.isEmpty(json)) {
            return null;
        }
        try {
            return objectMapper.readValue(json, clazz);
        } catch (Exception e) {
            throw new JSONException(e);
        }
    }

    /**
     * json字符串转对象(支持泛型)
     * @param json
     * @param targetType
     * @param <T>
     * @return
     */
    public <T> T parseObject(String json, TargetType<T> targetType) {
        if (StringUtils.isEmpty(json)) {
            return null;
        }
        try {
            return objectMapper.readValue(json, targetType);
        } catch (Exception e) {
            throw new JSONException(e);
        }
    }

    /**
     * json字符串转集合
     * @param json
     * @param collectionClazz
     * @param elementClazz
     * @param <E>
     * @return
     */
    public <E> Collection<E> parseCollection(String json, Class<? extends Collection> collectionClazz, Class<E> elementClazz) {
        if (StringUtils.isEmpty(json)) {
            return null;
        }
        CollectionType collectionType = objectMapper.getTypeFactory().constructCollectionType(collectionClazz, elementClazz);
        try {
            return objectMapper.readValue(json, collectionType);
        } catch (Exception e) {
            throw new JSONException(e);
        }
    }

    /**
     * json转Map对象
     * @param json
     * @param keyClazz
     * @param valueClazz
     * @param <K>
     * @param <V>
     * @return
     */
    public <K, V> Map<K, V> parseMap(String json, Class<K> keyClazz, Class<V> valueClazz) {
        if (StringUtils.isEmpty(json)) {
            return null;
        }
        MapType mapType = objectMapper.getTypeFactory().constructMapType(Map.class, keyClazz, valueClazz);
        try {
            return objectMapper.readValue(json, mapType);
        } catch (Exception e) {
            throw new JSONException(e);
        }
    }

    /**
     * 对象转json字符串
     * @param bean
     * @return
     */
    public String toJSONString(Object bean) {
        if (bean == null) {
            return null;
        }
        if (bean instanceof String) {
            return (String) bean;
        }
        try {
            return objectMapper.writeValueAsString(bean);
        } catch (Exception e) {
            throw new JSONException(e);
        }
    }

    /**
     * 利用Map构建json字符串
     * @param mapSupplier
     * @param <K>
     * @param <V>
     * @return
     */
    public <K, V> String buildJSON(Supplier<Map<K, V>> mapSupplier) {
        return toJSONString(mapSupplier.get());
    }

    /**
     * 查询节点值
     * @param jsonStr
     * @param fieldName
     * @return
     */
    public String findValueAsText(String jsonStr, String fieldName) {
        JsonNode jsonNode = null;
        try {
            jsonNode = objectMapper.readTree(jsonStr);
        } catch (Exception e) {
            throw new JSONException(e);
        }
        return Optional.ofNullable(jsonNode.findValue(fieldName))
                .map(JsonNode::asText)
                .orElse(null);
    }

    /**
     * 查询节点值
     * @param jsonStr
     * @param fieldName
     * @return
     */
    public List<String> findValuesAsText(String jsonStr, String fieldName) {
        JsonNode jsonNode = null;
        try {
            jsonNode = objectMapper.readTree(jsonStr);
        } catch (Exception e) {
            throw new JSONException(e);
        }
        return jsonNode.findValuesAsText(fieldName);
    }

}
