package com.hncy.lock.spring.boot.starter.distribute;

import com.hncy.lock.spring.boot.starter.core.Reentrant;
import com.hncy.redis.spring.boot.starter.core.RedisRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.util.StringUtils;

import java.time.Duration;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

/**
 * 基于Redis实现的可重入分布式锁
 * @Author: ying.dong
 * @Date: 2021/5/27 18:06
 */
@Slf4j
public class ReentrantRedisLock implements DLock, Reentrant {

    private final RedisRepository<String> redisRepository;

    private final ThreadLocal<String> context;

    /**
     * 自旋间隔时间(默认5毫秒)
     */
    private static final long DEFAULT_SPIN_DURATION = 5;

    /**
     * 自旋间隔时间
     */
    private final long spinDuration;

    /**
     * 默认锁释放阈值
     * 50毫秒
     */
    private static final long DEFAULT_UNLOCK_TIME_THRESHOLD = 50;

    /**
     * 调用unlock时，ttl时间大于此值的才会被立即释放，否则等待自动超时
     * 单位：毫秒
     */
    private final long unlockTimeThreshold;

    public ReentrantRedisLock(RedisRepository<String> redisRepository) {
        this(redisRepository, DEFAULT_SPIN_DURATION, DEFAULT_UNLOCK_TIME_THRESHOLD);
    }

    public ReentrantRedisLock(RedisRepository<String> redisRepository, long spinDuration) {
        this(redisRepository, spinDuration, DEFAULT_UNLOCK_TIME_THRESHOLD);
    }

    public ReentrantRedisLock(RedisRepository<String> redisRepository, long spinDuration, long unlockTimeThreshold) {
        this.redisRepository = redisRepository;
        this.context = new ThreadLocal<>();
        this.spinDuration = spinDuration;
        this.unlockTimeThreshold = unlockTimeThreshold;
    }

    @Override
    public <T> T lock(@NonNull String key, long wait, long lease, @NonNull Supplier<T> handler, Supplier<T> timeoutHandler) {
        return doLock(key, wait, lease, handler, timeoutHandler, false);
    }

    @Override
    public <T> T lock(@NonNull String key, long wait, long lease, boolean fastFail, @NonNull Supplier<T> handler, @Nullable Supplier<T> timeoutHandler) {
        return doLock(key, wait, lease, handler, timeoutHandler, true);
    }


    @Override
    public void unlock(String key) {
        try {
            long ttl = redisRepository.ttl(key, TimeUnit.MILLISECONDS);
            String cachedValue = redisRepository.get(key);
            if (StringUtils.isEmpty(cachedValue)) {
                log.warn("redis lock already released. key: {}", key);
                context.remove();
                return;
            }

            //ttl值大于指定值才会立即释放锁，避免误删（因为非原子操作）
            if (ttl < unlockTimeThreshold) {
                log.warn("waiting for release redis lock....., key: {}", key);
                context.remove();
                return;
            }

            if (cachedValue.equals(context.get())) {
                //属于自己的锁才释放
                context.remove();
                redisRepository.del(key);
                log.info("redis lock released. key: {}", key);
            }
        } catch (Exception e) {
            log.error("unlock redis error. key: {}", key, e);
        } finally {
            //避免污染
            context.remove();
        }
    }


    /**
     * 加锁 (锁会自动释放，无需显式Unlock)
     * @param key
     * @param wait 获取锁等待时间(秒)
     * @param lease 锁定时间(秒)
     * @param handler 获取到锁时执行业务逻辑
     * @param timeoutHandler 在wait结束时仍然没有获取到锁时的执行逻辑
     * @param fastFail 是否快速失败（true: 检查到锁存在，则直接return。 false: 等待锁，直到超时）
     */
    private  <T> T doLock(@NonNull String key, long wait, long lease, @NonNull Supplier<T> handler, Supplier<T> timeoutHandler, boolean fastFail) {
        long end = System.currentTimeMillis() + Duration.ofSeconds(wait).toMillis();

        //是否已经触发快速失败
        boolean doFastFail = false;

        try {
            if (!StringUtils.isEmpty(context.get())) {
                log.debug("reentrant lock. key: {}", context.get());
                //执行业务
                return handler.get();
            }

            String lockVal = UUID.randomUUID().toString();

            //自旋
            while (System.currentTimeMillis() < end) {
                if (!redisRepository.setNxEx(key, lockVal, lease)) {
                    if (fastFail) {
                        doFastFail = true;
                        log.warn("lock key [{}] exists. ignore....", key);
                        return null;
                    }
                    Thread.sleep(spinDuration);
                    continue;
                }
                context.set(lockVal);
                //执行业务
                log.debug("redis lock success. key: {}", key);
                return handler.get();
            }
            //执行超时逻辑
            log.warn("waiting for lock timeout. key: {}", key);
            if (timeoutHandler != null) {
                return timeoutHandler.get();
            }
            return null;
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } finally {
            if (!doFastFail) {
                unlock(key);
            }
        }
    }
}
