package com.hncy.redis.spring.boot.starter.core;

import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.DataType;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.*;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.lang.Nullable;

import java.time.Duration;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @Author: ying.dong
 * @Date: 2021/4/22 17:55
 */
public class StringRedisRepository implements RedisRepository<String> {

    private final StringRedisTemplate redisTemplate;

    public StringRedisRepository(StringRedisTemplate stringRedisTemplate) {
        this.redisTemplate = stringRedisTemplate;
    }

    public void set(String key, String value) {
        redisTemplate.opsForValue().set(key, value);
    }

    public String get(String key) {
        return redisTemplate.opsForValue().get(key);
    }

    public void setEx(String key, String value, long timeout) {
        redisTemplate.opsForValue().set(key, value, Duration.ofSeconds(timeout));
    }

    public Boolean setNx(String key, String value) {
        return redisTemplate.opsForValue().setIfAbsent(key, value);
    }

    public Boolean setNxEx(String key, String value, long timeout) {
        return redisTemplate.opsForValue().setIfAbsent(key, value, Duration.ofSeconds(timeout));
    }

    public Long ttl(String key) {
        return redisTemplate.opsForValue().getOperations().getExpire(key);
    }

    @Override
    public Long ttl(String key, TimeUnit timeUnit) {
        return redisTemplate.opsForValue().getOperations().getExpire(key, timeUnit);
    }

    public boolean del(String key) {
        return redisTemplate.delete(key);
    }

    public void hset(String key, String hashKey, String hashValue) {
        opsForHash().put(key, hashKey, hashValue);
    }

    public String hget(String key, String hashKey) {
        return opsForHash().get(key, hashKey);
    }

    public List<String> hMultiGet(String key, List<String> hashKeys) {
        return opsForHash().multiGet(key, hashKeys);
    }

    public Map<String, String> hgetAll(String key) {
        return opsForHash().entries(key);
    }

    public Long hdel(String key, String... hashKeys) {
        return opsForHash().delete(key, hashKeys);
    }

    public HashOperations<String, String, String> opsForHash() {
        return redisTemplate.opsForHash();
    }

    public ListOperations<String, String> opsForList() {
        return redisTemplate.opsForList();
    }

    public SetOperations<String, String> opsForSet() {
        return redisTemplate.opsForSet();
    }

    public Set<String> keys(String pattern) {
        return redisTemplate.keys(pattern);
    }

    //NONE("none"), STRING("string"), LIST("list"), SET("set"), ZSET("zset"), HASH("hash");
    public DataType type(String key) {
        return redisTemplate.type(key);
    }

    public List<String> lrange(final String key, final long start, final long stop) {
        return opsForList().range(key, start, stop);
    }

    public long llen(String key) {
        return opsForList().size(key);
    }

    public Set<String> smembers(String key) {
        return opsForSet().members(key);
    }

    public Boolean exists(String key) {
        return redisTemplate.opsForValue().getOperations().hasKey(key);
    }

    public void incr(String key) {
        redisTemplate.opsForValue().increment(key);
    }

    public long incrAndGet(String key) {
        return redisTemplate.opsForValue().increment(key);
    }

    public void decr(String key) {
        redisTemplate.opsForValue().decrement(key);
    }

    public void expire(String key, long timeout) {
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
    }

    public void expireAt(String key, Date date) {
        redisTemplate.expireAt(key, date);
    }

    public void rpush(String key, String value) {
        opsForList().rightPush(key, value);
    }

    public void setAdd(String key, String value) {
        opsForSet().add(key, value);
    }

    public String setPop(String key) {
        return opsForSet().pop(key);
    }

    public long setSize(String key) {
        return opsForSet().size(key);
    }

    public String lindex(String key, long index) {
        return opsForList().index(key, index);
    }

    @Override
    public List<Object> executePipelined(RedisConnectionConsumer<String, String> consumer) {
        return redisTemplate.executePipelined(new RedisCallback<Object>() {
            @Nullable
            @Override
            public Object doInRedis(RedisConnection connection) throws DataAccessException {
                RedisSerializer<String> keySerializer = (RedisSerializer<String>) redisTemplate.getKeySerializer();
                RedisSerializer<String> valueSerializer = (RedisSerializer<String>) redisTemplate.getValueSerializer();
                consumer.accept(connection, keySerializer, valueSerializer);
                return null;
            }
        });
    }

}
